package org.etsi.osl.tmf.gam673.api;


import io.swagger.v3.oas.annotations.Parameter;
import jakarta.validation.Valid;
import org.etsi.osl.tmf.gam673.model.GeographicAddress;
import org.etsi.osl.tmf.gam673.model.GeographicSubAddress;
import org.etsi.osl.tmf.gam673.reposervices.GeographicAddressManagementService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import java.util.List;
import java.util.Set;

@Controller
@RequestMapping("/geographicAddressManagement/v4/")
public class GeographicAddressManagementApiController implements GeographicAddressManagementApi{
    private static final Logger log = LoggerFactory.getLogger(GeographicAddressManagementApiController.class);

    private static final String COULD_NOT_SERIALIZE="Couldn't serialize response for content type application/json";
    private final GeographicAddressManagementService geographicAddressManagementService;

    @Autowired
    public GeographicAddressManagementApiController(GeographicAddressManagementService geographicAddressManagementService) {
        this.geographicAddressManagementService = geographicAddressManagementService;
    }

    @PreAuthorize("hasAnyAuthority('ROLE_USER')" )
    @Override
    public ResponseEntity<List<GeographicAddress>> listGeographicAddress() {
        try {
            return new ResponseEntity<>(geographicAddressManagementService.findAllGeographicAddress(), HttpStatus.OK);

        } catch (Exception e) {
            log.error(COULD_NOT_SERIALIZE, e);
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @PreAuthorize("hasAnyAuthority('ROLE_USER')" )
    @Override
    public ResponseEntity<GeographicAddress> createGeographicAddress(
            @Parameter(description = "The geographic address to be created", required = true) @Valid @RequestBody GeographicAddress geographicAddress
    ) {
        try{
            GeographicAddress c = geographicAddressManagementService.createGeographicAddress(geographicAddress);
            return new ResponseEntity<>(c, HttpStatus.OK);
        } catch (Exception e) {
            log.error(COULD_NOT_SERIALIZE, e);
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @PreAuthorize("hasAnyAuthority('ROLE_USER')" )
    @Override
    public ResponseEntity<Void> deleteGeographicAddress(
            @Parameter(description = "Identifier of the geographic address", required = true) @PathVariable("id") String id) {

        try {
            return new ResponseEntity<>(geographicAddressManagementService.deleteGeographicAddressById(id), HttpStatus.OK);
        }catch (Exception e) {
            log.error(COULD_NOT_SERIALIZE, e);
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }

    }

    @PreAuthorize("hasAnyAuthority('ROLE_USER')" )
    @Override
    public ResponseEntity<GeographicAddress> retrieveGeographicAddress(
            @Parameter(description = "Identifier of the geographic address", required = true) @PathVariable("id") String id) {
        try{
            GeographicAddress c = geographicAddressManagementService.findGeographicAddressByUUID(id);

            return new ResponseEntity<>(c, HttpStatus.OK);
        }catch (Exception e){
            log.error(COULD_NOT_SERIALIZE, e);
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @PreAuthorize("hasAnyAuthority('ROLE_USER')" )
    @Override
    public ResponseEntity<Set<GeographicSubAddress>> retrieveGeographicSubAddresses(
            @Parameter(description = "Identifier of the geographic address", required = true) @PathVariable("id") String id) {
        try{
            Set<GeographicSubAddress> c = geographicAddressManagementService.findAllGeographicSubAddresses(id);

            return new ResponseEntity<>(c, HttpStatus.OK);
        }catch (Exception e){
            log.error(COULD_NOT_SERIALIZE, e);
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }


    @PreAuthorize("hasAnyAuthority('ROLE_USER')" )
    @Override
    public ResponseEntity<GeographicAddress> patchGeographicAddress(
            @Parameter(description = "Identifier of the geographic address", required = true) @PathVariable("id") String id,
            @Parameter(description = "The Geographic Address to be updated", required = true) @Valid @RequestBody GeographicAddress geographicAddress) {
        try{
            GeographicAddress c = geographicAddressManagementService.updateGeographicAddress(id, geographicAddress);

            return new ResponseEntity<>(c, HttpStatus.OK);
        }catch (Exception e){
            log.error(COULD_NOT_SERIALIZE, e);
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @PreAuthorize("hasAnyAuthority('ROLE_USER')" )
    @Override
    public ResponseEntity<GeographicSubAddress> retrieveGeographicSubAddress(
            @Parameter(description = "Identifier of the geographic address", required = true) @PathVariable("geoAddressId") String geoAddressId,
            @Parameter(description = "Identifier of the geographic sub address", required = true) @PathVariable("geoSubAddressId") String geoSubAddressId){
        try{
            GeographicSubAddress c = geographicAddressManagementService.findGeographicSubAddressByUUID(geoAddressId,geoSubAddressId);

            return new ResponseEntity<>(c, HttpStatus.OK);
        }catch (Exception e){
            log.error(COULD_NOT_SERIALIZE, e);
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

}
