package org.etsi.osl.services.api.gam673;

import org.etsi.osl.tmf.gam673.api.GeographicAddressManagementApiController;
import org.etsi.osl.tmf.gam673.model.GeographicAddress;
import org.etsi.osl.tmf.gam673.model.GeographicSubAddress;
import org.etsi.osl.tmf.gam673.reposervices.GeographicAddressManagementService;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.doNothing;
import static org.mockito.Mockito.doThrow;
import static org.mockito.Mockito.when;

class GeographicAddressManagementControllerTest {

    @InjectMocks
    private GeographicAddressManagementApiController controller;


    @Mock
    private GeographicAddressManagementService service;

    @BeforeEach
    void setUp() {
        MockitoAnnotations.openMocks(this);
    }

    @Test
    void testRetrieveGeographicAddressByUUID() {
        GeographicAddress address = new GeographicAddress();
        // Add test data to address list
        when(service.findGeographicAddressByUUID("test")).thenReturn(address);

        ResponseEntity<GeographicAddress> response = controller.retrieveGeographicAddress("test");

        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertEquals(address, response.getBody());
    }

    @Test
    void throwExceptionTestWhenRetrieveGeographicAddressByUUID(){
        when(service.findGeographicAddressByUUID("test")).thenThrow(RuntimeException.class);
        ResponseEntity<GeographicAddress> response = controller.retrieveGeographicAddress("test");
        assertEquals(HttpStatus.INTERNAL_SERVER_ERROR, response.getStatusCode());

    }

    @Test
    void testRetrieveGeographicSubAddressByUUID() {
        GeographicSubAddress address = new GeographicSubAddress();
        // Add test data to address list
        when(service.findGeographicSubAddressByUUID("test","test")).thenReturn(address);

        ResponseEntity<GeographicSubAddress> response = controller.retrieveGeographicSubAddress("test","test");

        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertEquals(address, response.getBody());
    }

    @Test
    void throwExceptionTestWhenRetrieveGeographicSubAddressByUUID(){
        when(service.findGeographicSubAddressByUUID("test","test")).thenThrow(RuntimeException.class);
        ResponseEntity<GeographicSubAddress> response = controller.retrieveGeographicSubAddress("test","test");
        assertEquals(HttpStatus.INTERNAL_SERVER_ERROR, response.getStatusCode());

    }

    @Test
    void testRetrieveGeographicSubAddresses() {
        Set<GeographicSubAddress> subAddress = new HashSet<>();
        // Add test data to address list
        when(service.findAllGeographicSubAddresses("test")).thenReturn(subAddress);

        ResponseEntity<Set<GeographicSubAddress>> response = controller.retrieveGeographicSubAddresses("test");

        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertEquals(subAddress, response.getBody());
    }

    @Test
    void throwExceptionTestWhenRetrieveGeographicSubAddresses(){
        when(service.findAllGeographicSubAddresses("test")).thenThrow(RuntimeException.class);
        ResponseEntity<Set<GeographicSubAddress>> response = controller.retrieveGeographicSubAddresses("test");
        assertEquals(HttpStatus.INTERNAL_SERVER_ERROR, response.getStatusCode());

    }

    @Test
    void testRetrieveGeographicAddresses() {
        List<GeographicAddress> addresses = new ArrayList<>();
        // Add test data to address list
        when(service.findAllGeographicAddress()).thenReturn(addresses);

        ResponseEntity<List<GeographicAddress>> response = controller.listGeographicAddress();

        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertEquals(addresses, response.getBody());
    }

    @Test
    void throwExceptionTestWhenRetrieveGeographicAddresses(){
        when(service.findAllGeographicAddress()).thenThrow(RuntimeException.class);
        ResponseEntity<List<GeographicAddress>> response = controller.listGeographicAddress();
        assertEquals(HttpStatus.INTERNAL_SERVER_ERROR, response.getStatusCode());

    }
    @Test
    void testCreateGeographicAddress() {
        GeographicAddress address = new GeographicAddress();

        when(service.createGeographicAddress(any())).thenReturn(address);

        ResponseEntity<GeographicAddress> response = controller.createGeographicAddress(address);

        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertEquals(address, response.getBody());
    }

    @Test
    void testExceptionWhenCreateGeographicAddress(){
        GeographicAddress address = new GeographicAddress();

        when(service.createGeographicAddress(any())).thenThrow(RuntimeException.class);

        ResponseEntity<GeographicAddress> response = controller.createGeographicAddress(address);

        assertEquals(HttpStatus.INTERNAL_SERVER_ERROR, response.getStatusCode());
    }

    @Test
    void testDeleteGeographicAddress() {

        doNothing().when(service).deleteGeographicAddressById(anyString());
        ResponseEntity<Void> response = controller.deleteGeographicAddress("addressId");

        assertEquals(HttpStatus.OK, response.getStatusCode());
    }

    @Test
    void testDeleteGeographicAddressException() {

        doThrow(RuntimeException.class).when(service).deleteGeographicAddressById(anyString());
        ResponseEntity<Void> response = controller.deleteGeographicAddress("addressId");

        assertEquals(HttpStatus.INTERNAL_SERVER_ERROR, response.getStatusCode());
    }

    @Test
    void testPatchGeographicAddress() {
        String addressId = "addressId";
        GeographicAddress updatedAddress = new GeographicAddress();
        // Set up mock service behavior
        when(service.updateGeographicAddress(anyString(), any())).thenReturn(updatedAddress);

        ResponseEntity<GeographicAddress> response = controller.patchGeographicAddress(addressId, updatedAddress);

        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertEquals(updatedAddress, response.getBody());
    }

    @Test
    void testPatchGeographicAddressException() {
        String addressId = "addressId";
        GeographicAddress updatedAddress = new GeographicAddress();
        // Set up mock service behavior
        when(service.updateGeographicAddress(anyString(), any())).thenThrow(RuntimeException.class);

        ResponseEntity<GeographicAddress> response = controller.patchGeographicAddress(addressId, updatedAddress);

        assertEquals(HttpStatus.INTERNAL_SERVER_ERROR, response.getStatusCode());
    }

}
