package org.etsi.osl.services.api.gam673;

import org.etsi.osl.tmf.gam673.model.GeographicAddress;
import org.etsi.osl.tmf.gam673.model.GeographicSubAddress;
import org.etsi.osl.tmf.gam673.repo.GeographicAddressManagementRepository;
import org.etsi.osl.tmf.gam673.reposervices.GeographicAddressManagementService;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.BeforeEach;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.test.context.ActiveProfiles;

import java.util.List;
import java.util.Optional;
import java.util.Set;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertNull;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@AutoConfigureMockMvc
@ActiveProfiles("testing")
class GeographicAddressManagementServiceTest {

    @InjectMocks
    private GeographicAddressManagementService service;

    @Mock
    private GeographicAddressManagementRepository repository;

    @BeforeEach
    void setUp() {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    void testFindAllGeographicAddress() {
        // Mock data
        List<GeographicAddress> addresses = List.of(new GeographicAddress(), new GeographicAddress());
        when(repository.findAll()).thenReturn(addresses);

        List<GeographicAddress> result = service.findAllGeographicAddress();

        assertEquals(addresses, result);
    }

    @Test
    void testFindGeographicAddressByUUID() {
        // Mock data
        String uuid = "123";
        GeographicAddress address = new GeographicAddress();
        address.setUuid(uuid);
        when(repository.findByUuid(uuid)).thenReturn(Optional.of(address));

        GeographicAddress result = service.findGeographicAddressByUUID(uuid);

        assertNotNull(result);
        assertEquals(address, result);
    }

    @Test
    void testFindAllGeographicSubAddress() {
        Optional<GeographicAddress> address = Optional.of(new GeographicAddress());

        GeographicSubAddress subAddress = new GeographicSubAddress();
        subAddress.setSubAddressType("test");
        subAddress.setBuildingName("test");
        subAddress.setSubUnitType("test");

        address.get().setGeographicSubAddresses(Set.of(subAddress));

        // Mock data
        when(repository.findByUuid("test")).thenReturn(address);

        Set<GeographicSubAddress> result = service.findAllGeographicSubAddresses("test");

        assertEquals(address.get().getGeographicSubAddresses(), result);
    }

    @Test
    void testFindGeographicSubAddressByUUID() {
        // Mock data
        String addressId = "123";
        String subAddressId = "321";

        GeographicSubAddress subAddress = new GeographicSubAddress();
        subAddress.setSubAddressType("test");
        subAddress.setBuildingName("test");
        subAddress.setSubUnitType("test");
        subAddress.setUuid(subAddressId);

        GeographicAddress address = new GeographicAddress();
        address.setUuid(addressId);

        address.setGeographicSubAddresses(Set.of(subAddress));

        when(repository.findByUuid(addressId)).thenReturn(Optional.of(address));

        GeographicSubAddress result = service.findGeographicSubAddressByUUID(addressId, subAddressId);

        assertNotNull(result);
        assertEquals(subAddress, result);
    }

    @Test
    void testCreateGeographicAddress() {
        // Mock data
        GeographicAddress address = new GeographicAddress();
        when(repository.save(any())).thenReturn(address);

        GeographicAddress result = service.createGeographicAddress(address);

        assertEquals(address, result);
    }

    @Test
    void testUpdateGeographicAddress() {
        // Mock data
        String id = "123";
        GeographicAddress existingAddress = new GeographicAddress();
        GeographicAddress newAddress = new GeographicAddress();
        when(repository.findByUuid(id)).thenReturn(Optional.of(existingAddress));
        when(repository.save(any())).thenReturn(existingAddress);

        GeographicAddress result = service.updateGeographicAddress(id, newAddress);

        assertNotNull(result);
        assertEquals(existingAddress, result);
        // Add additional assertions for updated fields if needed
    }

    @Test
    void testDeleteGeographicAddressById() {
        // Mock data
        String id = "123";
        GeographicAddress existingAddress = new GeographicAddress();
        when(repository.findByUuid(id)).thenReturn(Optional.of(existingAddress));

        Void result = service.deleteGeographicAddressById(id);

        assertNull(result);
        verify(repository, times(1)).delete(existingAddress);
    }

}
